@@ -19,34 +19,43 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
+// own
 #include "app.h"
-#include "board.h"
-#include "prefs.h"
-#include "ui_settings.h"
-#include "kshisen_debug.h"
 
-#include <kmahjonggconfigdialog.h>
-#include <highscore/kscoredialog.h>
-#include <kstandardgameaction.h>
-
-#include <kactioncollection.h>
-#include <kconfig.h>
-#include <kconfigdialog.h>
-#include <klineedit.h>
-#include <KLocalizedString>
-#include <kmessagebox.h>
-#include <kseparator.h>
-#include <kshortcutsdialog.h>
-#include <kstandardaction.h>
-#include <kstandardguiitem.h>
-#include <ktoggleaction.h>
+// STL
+#include <cmath>
 
+// Qt
 #include <QIcon>
 #include <QPointer>
 #include <QStatusBar>
 #include <QTimer>
 
-#include <cmath>
+// KDE
+#include <KActionCollection>
+#include <KConfig>
+#include <KConfigDialog>
+#include <KLineEdit>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KSeparator>
+#include <KShortcutsDialog>
+#include <KStandardAction>
+#include <KStandardGuiItem>
+#include <KToggleAction>
+
+// KDEGames
+#include <KScoreDialog>
+#include <KStandardGameAction>
+
+// Kmahjongg
+#include <kmahjonggconfigdialog.h>
+
+// KShisen
+#include "board.h"
+#include "debug.h"
+#include "prefs.h"
+#include "ui_settings.h"
 
 
 /**
@@ -55,19 +64,20 @@
 class Settings : public QWidget, public Ui::Settings
 {
 public:
-    Settings(QWidget *parent)
-        : QWidget(parent) {
+    explicit Settings(QWidget * parent)
+        : QWidget(parent)
+    {
         setupUi(this);
     }
 };
 
-App::App(QWidget *parent)
-    : KXmlGuiWindow(parent),
-      m_gameTipLabel(nullptr),
-      m_gameTimerLabel(nullptr),
-      m_gameTilesLabel(nullptr),
-      m_gameCheatLabel(nullptr),
-      m_board(nullptr)
+App::App(QWidget * parent)
+    : KXmlGuiWindow(parent)
+    , m_gameTipLabel(nullptr)
+    , m_gameTimerLabel(nullptr)
+    , m_gameTilesLabel(nullptr)
+    , m_gameCheatLabel(nullptr)
+    , m_board(nullptr)
 {
     m_board = new Board(this);
     m_board->setObjectName(QStringLiteral("board"));
@@ -79,6 +89,7 @@ App::App(QWidget *parent)
     setupGUI();
 
     updateItems();
+    updateTileDisplay();
 }
 
 
@@ -112,7 +123,7 @@ void App::setupActions()
     KStandardGameAction::redo(this, SLOT(redo()), actionCollection());
     KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
 
-    KToggleAction *soundAction = new KToggleAction(QIcon::fromTheme(QStringLiteral("speaker")), i18n("Play Sounds"), this);
+    KToggleAction * soundAction = new KToggleAction(QIcon::fromTheme(QStringLiteral("speaker")), i18n("Play Sounds"), this);
     soundAction->setChecked(Prefs::sounds());
     actionCollection()->addAction(QStringLiteral("sounds"), soundAction);
     connect(soundAction, &KToggleAction::triggered, m_board, &Board::setSoundsEnabled);
@@ -128,12 +139,11 @@ void App::setupActions()
     connect(m_board, &Board::selectAMatchingTile, this, &App::notifySelectAMatchingTile);
     connect(m_board, &Board::selectAMove, this, &App::notifySelectAMove);
 
-    // TODO: Use QElapsedTime?
-    QTimer *timer = new QTimer(this);
+    auto timer = new QTimer(this);
     connect(timer, &QTimer::timeout, this, &App::updateTimeDisplay);
     timer->start(1000);
 
-    connect(m_board, &Board::changed, this, &App::updateTileDisplay);
+    connect(m_board, &Board::tileCountChanged, this, &App::updateTileDisplay);
     connect(m_board, &Board::endOfGame, this, &App::slotEndOfGame);
 
     connect(this, &App::invokeNewGame, m_board, &Board::newGame);
@@ -145,6 +155,7 @@ void App::newGame()
     setCheatModeEnabled(false);
     setPauseEnabled(false);
     updateItems();
+    updateTileDisplay();
 }
 
 void App::restartGame()
@@ -186,6 +197,7 @@ void App::undo()
     m_board->setGameStuckEnabled(false);
 
     updateItems();
+    updateTileDisplay();
 }
 
 void App::redo()
@@ -195,6 +207,7 @@ void App::redo()
     }
     m_board->redo();
     updateItems();
+    updateTileDisplay();
 }
 
 void App::hint()
@@ -240,10 +253,10 @@ void App::slotEndOfGame()
         m_board->setGameStuckEnabled(true);
     } else {
         m_board->setGameOverEnabled(true);
-        QString timeString = i18nc("time string: hh:mm:ss", "%1:%2:%3",
-                                   QString().sprintf("%02d", m_board->currentTime() / 3600),
-                                   QString().sprintf("%02d", (m_board->currentTime() / 60) % 60),
-                                   QString().sprintf("%02d", m_board->currentTime() % 60));
+        QString const timeString = i18nc("time string: hh:mm:ss", "%1:%2:%3",
+                                         QString().sprintf("%02d", m_board->currentTime() / 3600),
+                                         QString().sprintf("%02d", (m_board->currentTime() / 60) % 60),
+                                         QString().sprintf("%02d", m_board->currentTime() % 60));
         KScoreDialog::FieldInfo scoreInfo;
         scoreInfo[KScoreDialog::Score].setNum(score(m_board->xTiles(), m_board->yTiles(), m_board->currentTime(), m_board->gravityFlag()));
         scoreInfo[KScoreDialog::Time] = timeString;
@@ -256,21 +269,23 @@ void App::slotEndOfGame()
         } else {
             scoreInfo[KScoreDialog::Custom1] = i18n("No");
         }
-        scoreDialog->setConfigGroup(QStringLiteral("%1x%2").arg(sizeX[Prefs::size()]).arg(sizeY[Prefs::size()]));
+        QString const configGroup = QStringLiteral("%1x%2").arg(m_board->xTiles()).arg(m_board->yTiles());
+        scoreDialog->setConfigGroup(qMakePair(QByteArray(configGroup.toUtf8()), configGroup));
 
         if (m_board->hasCheated()) {
-            QString message = i18n("\nYou could have been in the highscores\nif you did not use Undo or Hint.\nTry without them next time.");
+            QString const message = i18n("\nYou could have been in the highscores\nif you did not use Undo or Hint.\nTry without them next time.");
             KMessageBox::information(this, message, i18n("End of Game"));
         } else {
             if (scoreDialog->addScore(scoreInfo) > 0) {
-                QString message = i18n("Congratulations!\nYou made it into the hall of fame.");
+                QString const message = i18n("Congratulations!\nYou made it into the hall of fame.");
                 scoreDialog->setComment(message);
                 scoreDialog->exec();
             } else {
-                QString message = i18nc("%1 - time string like hh:mm:ss", "You made it in %1", timeString);
+                QString const message = i18nc("%1 - time string like hh:mm:ss", "You made it in %1", timeString);
                 KMessageBox::information(this, message, i18n("End of Game"));
             }
         }
+        delete scoreDialog;
     }
     updateItems();
 }
@@ -281,26 +296,20 @@ void App::updateTimeDisplay()
         return;
     }
     //qCDebug(KSHISEN_LOG) << "Time: " << m_board->currentTime();
-    int currentTime = m_board->currentTime();
-    QString message = i18n("Your time: %1:%2:%3 %4",
-                           QString().sprintf("%02d", currentTime / 3600),
-                           QString().sprintf("%02d", (currentTime / 60) % 60),
-                           QString().sprintf("%02d", currentTime % 60),
-                           m_board->isPaused() ? i18n("(Paused) ") : QString());
+    int const currentTime = m_board->currentTime();
+    QString const message = i18n("Your time: %1:%2:%3 %4",
+                                 QString().sprintf("%02d", currentTime / 3600),
+                                 QString().sprintf("%02d", (currentTime / 60) % 60),
+                                 QString().sprintf("%02d", currentTime % 60),
+                                 m_board->isPaused() ? i18n("(Paused) ") : QString());
 
     m_gameTimerLabel->setText(message);
-    // FIXME: temporary hack until I find out why m_board->tilesLeft() in updateTileDisplay() counts the previous state of the board, not the current (schwarzer)
-    updateTileDisplay();
 }
 
 void App::updateTileDisplay()
 {
-    int numberOfTiles = (m_board->xTiles() * m_board->yTiles());
-    QString message = i18n("Removed: %1/%2 ",
-                           QString().sprintf("%d", numberOfTiles - m_board->tilesLeft()),
-                           QString().sprintf("%d", numberOfTiles));
-
-    m_gameTilesLabel->setText(message);
+    int const numberOfTiles = m_board->tiles();
+    m_gameTilesLabel->setText(i18n("Removed: %1/%2 ", numberOfTiles - m_board->tilesLeft(), numberOfTiles));
 }
 
 void App::updateCheatDisplay()
@@ -310,17 +319,16 @@ void App::updateCheatDisplay()
 
 int App::score(int x, int y, int seconds, bool gravity) const
 {
-    double ntiles = x * y;
-    double tilespersec = ntiles / static_cast<double>(seconds);
+    double const ntiles = x * y;
+    double const tilespersec = ntiles / static_cast<double>(seconds);
 
-    double sizebonus = std::sqrt(ntiles / static_cast<double>(14.0 * 6.0));
-    double points = tilespersec / 0.14 * 100.0;
+    double const sizebonus = std::sqrt(ntiles / static_cast<double>(14.0 * 6.0));
+    double const points = tilespersec / 0.14 * 100.0;
 
     if (gravity) {
         return static_cast<int>(2.0 * points * sizebonus);
-    } else {
-        return static_cast<int>(points * sizebonus);
     }
+    return static_cast<int>(points * sizebonus);
 }
 
 void App::notifySelectATile()
@@ -358,6 +366,8 @@ void App::showHighscores()
 {
     KScoreDialog scoreDialog(KScoreDialog::Name | KScoreDialog::Time, this);
     scoreDialog.addField(KScoreDialog::Custom1, i18n("Gravity"), QStringLiteral("gravity"));
+    QString const configGroup = QStringLiteral("%1x%2").arg(m_board->xTiles()).arg(m_board->yTiles());
+    scoreDialog.setConfigGroup(qMakePair(QByteArray(configGroup.toUtf8()), configGroup));
     scoreDialog.exec();
 }
 
@@ -373,11 +383,11 @@ void App::showSettingsDialog()
     }
 
     //Use the classes exposed by LibKmahjongg for our configuration dialog
-    KMahjonggConfigDialog *dialog = new KMahjonggConfigDialog(this, QStringLiteral("settings"), Prefs::self());
-    dialog->addPage(new Settings(0), i18n("General"), QStringLiteral("games-config-options"));
+    KMahjonggConfigDialog * dialog = new KMahjonggConfigDialog(this, QStringLiteral("settings"), Prefs::self());
+    dialog->addPage(new Settings(nullptr), i18n("General"), QStringLiteral("games-config-options"));
     dialog->addTilesetPage();
     dialog->addBackgroundPage();
-    //dialog->setHelp(QString(), "kshisen");
+
     connect(dialog, &KMahjonggConfigDialog::settingsChanged, m_board, &Board::loadSettings);
     dialog->show();
 }
